#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#############################################################################################
# This plugin checks the status of Windows Defender and System Center Endpoint Protection.  #
# It creates one service called "AV state (Defender)" or "AV state (SCEP)".                 #
# The service will be in WARN/CRIT state, if one of the AV service are disabled, some       #
# signatures are outdated or the last quick scan is too old.                                # 
# ----------------------------------------------------------------------------------------- #
# Author: Michael Dvorak - specialis IT GmbH                                                #
# Version: 1.0 - 2021-12-26 - initial version                                               #
#############################################################################################

# Example of section data:
# <<<spit_defender_status>>>
# 10 1.1.18800.4 4.18.2111.5 Normal True 4.18.2111.5 True 1 25.12.2021-12:53:18 1.355.847.0 True 1 25.12.2021-12:53:21 1.355.847.0 True 05F2F8C2-C50A-49E7-BA78-34373E287263 0 N/A 01.01.1601-01:00:00 N/A 4294967295 - - True False True 0 2 True 1.1.18800.4 1 25.12.2021-12:53:21 1.355.847.0 True - 0 26.12.2021-05:47:41 26.12.2021-05:46:13 True 0 N/A N/A N/A N/A

from .agent_based_api.v1 import *
import pprint

def discover_spit_defender_state(section):
    if int(section[0][0]) == 6:
        avname="SCEP"
    else:
        avname="Defender"
    yield Service(item=avname)

def check_spit_defender_state(item,params,section):
    
    AMEngineVersion = section[0][1]
    AMProductVersion = section[0][2]
    AMServiceEnabled = section[0][3]
    AMServiceVersion = section[0][4]
    AntispywareEnabled = section[0][5]
    AntispywareSignatureAge = int(section[0][6])
    AntispywareSignatureLastUpdated = section[0][7].replace('-',' ')
    AntispywareSignatureVersion = section[0][8]
    AntivirusEnabled = section[0][9]
    AntivirusSignatureAge = int(section[0][10])
    AntivirusSignatureLastUpdated = section[0][11].replace('-',' ')
    AntivirusSignatureVersion = section[0][12]
    BehaviorMonitorEnabled = section[0][13]
    IoavProtectionEnabled = section[0][14]
    NISEnabled = section[0][15]
    NISEngineVersion = section[0][16]
    NISSignatureAge = int(section[0][17])
    NISSignatureLastUpdated = section[0][18].replace('-',' ')
    NISSignatureVersion = section[0][19]
    OnAccessProtectionEnabled = section[0][20]
    QuickScanAge = int(section[0][21])
    QuickScanEndTime = section[0][22].replace('-',' ')
    QuickScanStartTime = section[0][23].replace('-',' ')
    RealTimeProtectionEnabled = section[0][24]
    
    AntispywareWarnAge = params["signature_age"][0]
    AntispywareCritAge = params["signature_age"][1]
    AntivirusWarnAge = params["signature_age"][0]
    AntivirusCritAge = params["signature_age"][1]
    NISWarnAge = params["signature_age"][0]
    NISCritAge = params["signature_age"][1]
    QuickWarnAge = params["quickscan_age"][0]
    QuickCritAge = params["quickscan_age"][1]
    
    disabledServices = []
    enabledServices = []
    
    #check if all services are enabled
    if AMServiceEnabled != 'True':
        disabledServices.append('AMService')
    else:
        enabledServices.append('AMService')
        
    if AntispywareEnabled != 'True':
        disabledServices.append('Antispyware')
    else:
        enabledServices.append('Antispyware')
        
    if BehaviorMonitorEnabled != 'True':
        disabledServices.append('BehaviorMonitor')
    else:
        enabledServices.append('BehaviorMonitor')
        
    if IoavProtectionEnabled != 'True':
        disabledServices.append('IoavProtection')
    else:
        enabledServices.append('IoavProtection')
        
    if NISEnabled != 'True':
        disabledServices.append('NetworkInspectionSystem (NIS)')
    else:
        enabledServices.append('NetworkInspectionSystem (NIS)')
        
    if OnAccessProtectionEnabled != 'True':
        disabledServices.append('OnAccessProtection')
    else:
        enabledServices.append('OnAccessProtection')
        
    if RealTimeProtectionEnabled != 'True':
        disabledServices.append('RealTimeProtection')
    else:
        enabledServices.append('RealTimeProtection')
    
    enabledCount = len(enabledServices)
    servicesOnline = ','.join(enabledServices)
    
    mess_detail_info = []
    
    mess_detail_info.append("------------------------------------------------------------------------------------------------------------")
    mess_detail_info.append(f"Antimalware Clientversion: {AMProductVersion}")
    mess_detail_info.append(f"Module Version: {AMEngineVersion}")
    mess_detail_info.append(f"NIS Engine Version: {NISEngineVersion}")
    mess_detail_info.append("-----------------------------------------------------------")
    mess_detail_info.append(f"Enabled services: {enabledCount}/7 - {enabledServices}\n")
    
    mess_det = '\n'.join(mess_detail_info)
    
    if len(disabledServices)>0:
        s = State.CRIT
        servicesCount = len(disabledServices)
        servicesOffline = ','.join(disabledServices)
        mess_sum = f"Following services are disabled: {servicesCount}/7 - {servicesOffline}"
        yield Result(state=s,summary=mess_sum,details=mess_det)
    else:
        s = State.OK
        mess_sum = f"All services are enabled (7/7)"
        yield Result(state=s,summary=mess_sum,details=mess_det)
    
    signaturesOutdated = []
    
    # check if some signatures are outdated
    if AntispywareSignatureAge >= AntispywareCritAge:
        s = State.CRIT
        signaturesOutdated.append(f"Antispyware ({AntispywareSignatureAge}days)")
    elif AntispywareSignatureAge >= AntispywareWarnAge:
        s = State.WARN
        signaturesOutdated.append(f"Antispyware ({AntispywareSignatureAge}days)")
        
    if AntivirusSignatureAge >= AntivirusCritAge:
        s = State.CRIT
        signaturesOutdated.append(f"Antivirus ({AntivirusSignatureAge}days)")
    elif AntivirusSignatureAge >= AntivirusWarnAge:
        if s != State.CRIT:
            s = State.WARN
        signaturesOutdated.append(f"Antivirus ({AntivirusSignatureAge}days)")
        
    if NISSignatureAge >= NISCritAge:
        s = State.CRIT
        signaturesOutdated.append(f"Network Inspection System ({NISSignatureAge}days)")
    elif NISSignatureAge >= NISWarnAge:
        if s != State.CRIT:
            s = State.WARN
        signaturesOutdated.append(f"Network Inspection System ({NISSignatureAge}days)")
    
    mess_detail_info = []
    
    mess_detail_info.append("------------------------------------------------------------------------------------------------------------")
    mess_detail_info.append(f"Antispyware Signature Last Updated: {AntispywareSignatureLastUpdated}")
    mess_detail_info.append(f"Antispyware Signature Age: {AntispywareSignatureAge} days")
    mess_detail_info.append(f"Antispyware Signature Version: {AntispywareSignatureVersion}")
    mess_detail_info.append("-----------------------------------------------------------")
    mess_detail_info.append(f"Antivirus Signature Last Updated: {AntivirusSignatureLastUpdated}")
    mess_detail_info.append(f"Antivirus Signature Age: {AntivirusSignatureAge} days")
    mess_detail_info.append(f"Antivirus Signature Version: {AntivirusSignatureVersion}")
    mess_detail_info.append("-----------------------------------------------------------")
    mess_detail_info.append(f"NIS Signature Last Updated: {NISSignatureLastUpdated}")
    mess_detail_info.append(f"NIS Signature Age: {NISSignatureAge} days")
    mess_detail_info.append(f"NIS Signature Version: {NISSignatureVersion}")
    
    mess_det = '\n'.join(mess_detail_info)
    
    if len(signaturesOutdated)>0:
        outdatedCount = len(signaturesOutdated)
        outdatedSignatures = ','.join(signaturesOutdated)
        mess_sum = f"Following signatures are outdated: {outdatedCount}/3 - {outdatedSignatures}"
        yield Result(state=s,summary=mess_sum,details=mess_det)
    else:
        s = State.OK
        mess_sum = f"All signatures are up to date (3/3)"
        yield Result(state=s,summary=mess_sum,details=mess_det)
    
    mess_detail_info = []
    
    mess_detail_info.append("------------------------------------------------------------------------------------------------------------")
    mess_detail_info.append(f"Quick Scan Age: {QuickScanAge} days")
    mess_detail_info.append(f"Quick Scan Start Time: {QuickScanStartTime}")
    mess_detail_info.append(f"Quick Scan End Time: {QuickScanEndTime}")
   
    mess_det = '\n'.join(mess_detail_info)
    
    mess_sum = f"Last QuickScan is too old: {QuickScanAge}days ago - Last start: {QuickScanStartTime} - Last end: {QuickScanEndTime}"
    
    # check if last quickscan is outdated
    if QuickScanAge >= QuickCritAge:
        s = State.CRIT
        yield Result(state=s,summary=mess_sum,details=mess_det)
    elif QuickScanAge >= QuickWarnAge:
        s = State.WARN
        yield Result(state=s,summary=mess_sum,details=mess_det)
    else:
        s = State.OK
        mess_sum = f"Last QuickScan was OK"
        yield Result(state=s,summary=mess_sum,details=mess_det)
    
register.check_plugin(
    name="spit_defender_state",
    service_name="AV state (%s)",
    discovery_function=discover_spit_defender_state,
    check_function=check_spit_defender_state,
    check_default_parameters={"signature_age": [3,5],"quickscan_age": [2,3]},
    check_ruleset_name="spit_defender_state",
)
